;
;   Oil rig macro for use with SCASM 2.07 and higher
;
;   Parameters:
;   %1  Latitude
;   %2  Longitude
;   %3  Range (ignored)
;   %4  Scale (ignored)
;   %5  Heading
;   %6  up1 crane 1(left) heading angle
;   %7  up2 crane 1 elevation angle
;   %8  up3 crane 2(left) heading angle
;   %9  up4 crane 2 elevation angle

mif( [ $Version < 207 ] )
    error( you need at least SCASM version 2.07 )
mifend

;
; these are global variables because they are also used for
; section 16 code
;
Uvar( $ch 120 )          ; column height
Uvar( $mt  16 )          ; main deck thickness
UVar( $hm [$ch + $mt] )  ; main deck elevation
Uvar( $hb1 64 )          ; height of the main building with heli deck
Uvar( $he [$hm + $hb1] ) ; heli deck elevation

Area( 5 %1 %2 15 )
    ; some local variables
    UVar( $rb  48 )          ; big column radius
    UVar( $rs  22 )          ; small column radius
    Uvar( $rc  24 )          ; radius of crane column (3m)

    PerspectiveCall( :A )
    Jump( :end )

    Dbx( 28 43 29 )
    Dba( by Manfred Moldenhauer )

:A
    Perspective
    RefPoint( ns :p_ret %1 %2 )
    SetScaleX( :p_ret 0 0 13 )
    RotatedCall( :B 0 0 %5 )
    Return

:B
    ; Note! commented point numbers are element relative.
    ;
    Uvar( $ds  16 ) ; first point of main deck structure
    Uvar( $hd  48 ) ; first point of main building and heli deck
    Uvar( $b2  64 ) ; first point of 2nd (small) building
    Uvar( $tb  72 ) ; first point of the tower base
    Uvar( $dr  80 ) ; first point of the drilling derrick
    Uvar( $cr 114 ) ; first point of crane definition
    ;
    Points( $ds
         260 $ch  352    300 $ch  392 ; 0 1
         340 $ch  352    300 $ch  312 ; 2 3
         300 $ch -312    340 $ch -352 ; 4 5
         300 $ch -392    260 $ch -352 ; 6 7
        -260 $ch -352   -300 $ch -392 ; 8 9
        -340 $ch -352   -300 $ch -312 ; 10 11
        -300 $ch  312   -340 $ch  352 ; 12 13
        -300 $ch  392   -260 $ch  352 ; 14 15

         260 $hm  352    300 $hm  392 ; 16 17
         340 $hm  352    300 $hm  312 ; 18 19
         300 $hm -312    340 $hm -352 ; 20 21
         300 $hm -392    260 $hm -352 ; 22 23
        -260 $hm -352   -300 $hm -392 ; 24 25
        -340 $hm -352   -300 $hm -312 ; 26 27
        -300 $hm  312   -340 $hm  352 ; 28 29
        -300 $hm  392   -260 $hm  352 ; 30 31

        ; building with heli deck
         272 $hm -108    272 $hm -324 ;  0  1
        -272 $hm -324   -272 $hm -108 ;  2  3
         272 $he -108    272 $he -324 ;  4  5
        -272 $he -324   -272 $he -108 ;  6  7
         -56 $he -324   -112 $he -404 ;  8  9 the heli deck itself
        -272 $he -404   -332 $he -324 ; 10 11
        -332 $he -204   -272 $he -108 ; 12 13
        -112 $he -108    -56 $he -164 ; 14 15

        ; 2nd building
         200 $hm         320    200 $hm         160 ; 0 1
        -200 $hm         160   -200 $hm         320 ; 2 3
         200 [$hm + 32]  320    200 [$hm + 32]  160 ; 4 5
        -200 [$hm + 32]  160   -200 [$hm + 32]  320 ; 6 7

        ; tower base
          84 $hm   84            84 $hm  -84 ;  0  1
         -84 $hm  -84           -84 $hm   84 ;  2  3
          84 $he   84            84 $he  -84 ;  4  5
         -84 $he  -84           -84 $he   84 ;  6  7

        ; derrick
         50  $he          50     50   $he  -50 ; 0 1
        -50  $he         -50    -50   $he   50 ; 2 3
         40  [$he + 160]  40 ; 4
         40  [$he + 160]   0 ; 5
         40  [$he + 160] -40 ; 6
          0  [$he + 160] -40 ; 7
        -40  [$he + 160] -40 ; 8
        -40  [$he + 160]   0 ; 9
        -40  [$he + 160]  40 ; 10
          0  [$he + 160]  40 ; 11 ----
         40  [$he + 200]  40 ; 12
         40  [$he + 200] -40 ; 13
        -40  [$he + 200] -40 ; 14
        -40  [$he + 200]  40 ; 15 ----
         30  [$he + 200]  30 ; 16
         30  [$he + 200] -30 ; 17
        -30  [$he + 200] -30 ; 18
        -30  [$he + 200]  30 ; 19 ----
         25  [$he + 280]  25 ; 20
         25  [$he + 280] -25 ; 21
        -25  [$he + 280] -25 ; 22
        -25  [$he + 280]  25 ; 23 ----
         20  [$he + 360]  20 ; 24
         20  [$he + 360] -20 ; 25
        -20  [$he + 360] -20 ; 26
        -20  [$he + 360]  20 ; 27 ----
         20  [$he + 400]  20 ; 28
         20  [$he + 400] -20 ; 29
        -20  [$he + 400] -20 ; 30
        -20  [$he + 400]  20 ; 31
          0  [$he + 400]   0 ; 32
          0  [$he + 416]   0 ; 33
    )
    ;
    ; the elements, sections and sub-sections are sorted to avoid
    ; the bleed through effects
    ;
    VectorJump( :C m 0 32767 0  $ch )
    Call( :columns )
    Call( :deck )
    Return

:C
    Call( :deck )
    Call( :columns )
:p_ret
    Return

:deck
    VectorJump( :d2 m 0 32767 0  $hm )
    Call( :main_deck )
    Call( :buildings )
    Return

:d2
    Call( :buildings )
    Call( :main_deck )
    Return

:buildings
    VectorJump( :secD m 0 0 32767  -120 )
    ; midle & north
    Call( :heli_deck ) ; secD
    Call( :secABC )
    Return

:secD
    Call( :secABC )
    Call( :heli_deck ) ; secD
    Return

:secBE
    VectorJump( :secE  p 0 0  108 )
    Call( :tower )
    Call( :bldg2 )
    Return

:secE
    Call( :bldg2 )
    Call( :tower )
    Return

:secABC
    VectorJump( :secC m -32767 0 0   -200 )
    VectorJump( :secA m  32767 0 0   -200 )
    ; secB
    SetVar( 314 round[16384/90 * %7 * -1] )
    TransformCall( :crane [-300 + $rc] $hm 112  0 0  0 0   %6 0 )
    SetVar( 314 [round[16384/90 * %9 * -1]] )
    TransformCall( :crane [ 300 - $rc] $hm 112  0 0  0 0   %8 0 )
    Call( :secBE )
    Return

:secA
    SetVar( 314 [round[16384/90 * %9 * -1]] )
    TransformCall( :crane [ 300 - $rc] $hm 112  0 0  0 0   %8 0 )
    Call( :secBE )
    SetVar( 314 round[16384/90 * %7 * -1] )
    TransformCall( :crane [-300 + $rc] $hm 112  0 0  0 0   %6 0 )
    Return

:secC
    SetVar( 314 round[16384/90 * %7 * -1] )
    TransformCall( :crane [-300 + $rc] $hm 112  0 0  0 0   %6 0 )
    Call( :secBE )
    SetVar( 314 [round[16384/90 * %9 * -1]] )
    TransformCall( :crane [ 300 - $rc] $hm 112  0 0  0 0   %8 0 )
    Return

;
; The small building
; (machine area and pipe rack)
;
:bldg2
    SurfaceColor( 3 F0 )
    Bitmap( side8.r8 0 0 0 0 )
    TexPolyShading( a [$b2+0] [$b2+1] [$b2+5] [$b2+4] )
    Inst_7D
    TexPoly( a
        [$b2+5]  0 10
        [$b2+4] 10 10
        [$b2+0] 10 13
        [$b2+1]  0 13
    )
    TexPolyShading( ai [$b2+6] [$b2+5] [$b2+1] [$b2+2] )
    Inst_7D
    TexPoly( ai
        [$b2+6]  0 10
        [$b2+5] 25 10
        [$b2+1] 25 13
        [$b2+2]  0 13
    )
    TexPolyShading( a [$b2+7] [$b2+6] [$b2+2] [$b2+3] )
    Inst_7D
    TexPoly( a
        [$b2+7]  0 10
        [$b2+6] 10 10
        [$b2+2] 10 13
        [$b2+3]  0 13
    )
    TexPolyShading( a  [$b2+4] [$b2+7] [$b2+3] [$b2+0] )
    Inst_7D
    TexPoly( a
        [$b2+4]  0  2
        [$b2+7] 25  2
        [$b2+3] 25  5
        [$b2+0]  0  5
    )
    IfVarRange( :bldg22 346 2 4 )
    ; crash detection
    IfVarRange( :b2nc 382 $hm           [ipt[[$b2+4]z]-1] )
    IfVarRange( :b2nc 37E ipt[[$b2+3]x]  ipt[[$b2+0]x]    )
    IfVarRange( :b2nc 386 ipt[[$b2+1]y]  ipt[[$b2+0]y]    )
    SetVar    ( 284   6 )
:b2nc
    Bitmap( concrete.r8 0 0 0 0 )
:bldg22
    Poly( a  [$b2+4] [$b2+5] [$b2+6] [$b2+7] )
    ;
    IfVarAnd  ( :bdg2e 28C 6 )
    IfVarRange( :bdg2e 33B 0 900 )
    LineColor( F F0 )
    VectorJump( :bdg23 a  [$b2+ 0] [$b2+ 1] [$b2+ 5] [$b2+ 4] )
    DotPt( [$b2+ 4] )
    DotPt( [$b2+ 5] )
:bdg23
    VectorJump( :bdg24 ai  [$b2+ 6] [$b2+ 5] [$b2+ 1] [$b2+ 2] )
    DotPt( [$b2+ 5] )
    DotPt( [$b2+ 6] )
:bdg24
    VectorJump( :bdg25 a  [$b2+ 7] [$b2+ 6] [$b2+ 2] [$b2+ 3] )
    DotPt( [$b2+ 6] )
    DotPt( [$b2+ 7] )
:bdg25
    VectorJump( :bdg2e a  [$b2+ 4] [$b2+ 7] [$b2+ 3] [$b2+ 0] )
    DotPt( [$b2+ 4] )
    DotPt( [$b2+ 7] )
:bdg2e
    Return

:main_deck
    SurfaceColor( A F0 )
    VectorJump( :right p 0 90 0 )
    Poly( ai [$ds+ 2] [$ds+ 3] [$ds+19] [$ds+18] )
    Poly( ai [$ds+ 4] [$ds+ 5] [$ds+21] [$ds+20] )
    Poly( a  [$ds+ 6] [$ds+ 7] [$ds+23] [$ds+22] )
    Poly( a  [$ds+ 0] [$ds+ 1] [$ds+17] [$ds+16] )
    Poly( ai [$ds+12] [$ds+13] [$ds+29] [$ds+28] )
    Poly( ai [$ds+10] [$ds+11] [$ds+27] [$ds+26] )
    Poly( a  [$ds+ 8] [$ds+ 9] [$ds+25] [$ds+24] )
    Poly( a  [$ds+14] [$ds+15] [$ds+31] [$ds+30] )
    Jump( :md2 )

:right
    Poly( ai [$ds+12] [$ds+13] [$ds+29] [$ds+28] )
    Poly( ai [$ds+10] [$ds+11] [$ds+27] [$ds+26] )
    Poly( a  [$ds+ 8] [$ds+ 9] [$ds+25] [$ds+24] )
    Poly( a  [$ds+14] [$ds+15] [$ds+31] [$ds+30] )
    Poly( ai [$ds+ 2] [$ds+ 3] [$ds+19] [$ds+18] )
    Poly( ai [$ds+ 4] [$ds+ 5] [$ds+21] [$ds+20] )
    Poly( a  [$ds+ 6] [$ds+ 7] [$ds+23] [$ds+22] )
    Poly( a  [$ds+ 0] [$ds+ 1] [$ds+17] [$ds+16] )
:md2
    Poly( a [$ds+ 3] [$ds+ 4] [$ds+20] [$ds+19] )
    Poly( a [$ds+ 7] [$ds+ 8] [$ds+24] [$ds+23] )
    Poly( a [$ds+11] [$ds+12] [$ds+28] [$ds+27] )
    Poly( a [$ds+15] [$ds+ 0] [$ds+16] [$ds+31] )

    IfVarAnd  ( :md24 28C 6 )
    IfVarRange( :md24 33B 0 1000 )
    LineColor( F F0 )
    VectorJump( :md21 a [$ds+ 3] [$ds+ 4] [$ds+20] )
    DotLine( ipt[[$ds+19]x] [ipt[[$ds+19]z] -1] ipt[[$ds+19]y]
             ipt[[$ds+20]x] [ipt[[$ds+20]z] -1] ipt[[$ds+20]y] 6 )
:md21
    VectorJump( :md22 a [$ds+ 7] [$ds+ 8] [$ds+24] )
    DotLine( ipt[[$ds+23]x] [ipt[[$ds+23]z] -1] ipt[[$ds+23]y]
             ipt[[$ds+24]x] [ipt[[$ds+24]z] -1] ipt[[$ds+24]y] 5 )
:md22
    VectorJump( :md23 a [$ds+11] [$ds+12] [$ds+28] )
    DotLine( ipt[[$ds+27]x] [ipt[[$ds+27]z] -1] ipt[[$ds+27]y]
             ipt[[$ds+28]x] [ipt[[$ds+28]z] -1] ipt[[$ds+28]y] 6 )
:md23
    VectorJump( :md24 a [$ds+15] [$ds+ 0] [$ds+16] )
    DotLine( ipt[[$ds+31]x] [ipt[[$ds+31]z] -1] ipt[[$ds+31]y]
             ipt[[$ds+16]x] [ipt[[$ds+16]z] -1] ipt[[$ds+16]y] 5 )

:md24
    Poly( a [$ds+ 1] [$ds+ 2] [$ds+18] [$ds+17] )
    Poly( a [$ds+ 5] [$ds+ 6] [$ds+22] [$ds+21] )
    Poly( a [$ds+ 9] [$ds+10] [$ds+26] [$ds+25] )
    Poly( a [$ds+13] [$ds+14] [$ds+30] [$ds+29] )
    SurfaceColor( 3 F0 )
    ConcavePoly
    Poly( ad
      [$ds+0] [$ds+1] [$ds+2]  [$ds+3]  [$ds+4]  [$ds+5]  [$ds+6]  [$ds+7]
      [$ds+8] [$ds+9] [$ds+10] [$ds+11] [$ds+12] [$ds+13] [$ds+14] [$ds+15] )
    SurfaceColor( 3 F0 )
    ConcavePoly
    Poly( a
      [$ds+16] [$ds+17] [$ds+18] [$ds+19] [$ds+20] [$ds+21] [$ds+22] [$ds+23]
      [$ds+24] [$ds+25] [$ds+26] [$ds+27] [$ds+28] [$ds+29] [$ds+30] [$ds+31] )
    IfVarRange( :mde 382 $ch  [$hm-1] )
    IfVarRange( :mde 37E ipt[[$ds+12]x] ipt[[$ds+ 3]x] )
    IfVarRange( :mde 386 ipt[[$ds+ 7]y] ipt[[$ds+ 0]y] )
    SetVar( 284 6 )
:mde
    Return

;--------------------------
; heli deck drawing routine
;
:heli_deck
    VectorJump( :hd_low m 0 32737 0  $he )
    Call ( :hd_base )
    Call ( :hd_top  )
    Return

:hd_low
    Call ( :hd_top  )
    Call ( :hd_base )
    Return

:hd_base
    SurfaceColor( 3 F0 )
    Poly( ad [$hd+8] [$hd+9] [$hd+10] [$hd+11] [$hd+12] [$hd+13] )
    Bitmap( side8.r8 0 0 0 0 )
    TexPolyShading( a [$hd+0] [$hd+1] [$hd+5] [$hd+4] )
    Inst_7D
    TexPoly( a
        [$hd+5]  0  2
        [$hd+4] 13  2
        [$hd+0] 13  7
        [$hd+1]  0  7
    )
    TexPolyShading( a [$hd+6] [$hd+5] [$hd+1] [$hd+2] )
    Inst_7D
    TexPoly( a
        [$hd+6]  0  2
        [$hd+5] 34  2
        [$hd+1] 34  7
        [$hd+2]  0  7
    )
    TexPolyShading( a [$hd+7] [$hd+6] [$hd+2] [$hd+3] )
    Inst_7D
    TexPoly( a
        [$hd+7]  0  2
        [$hd+6] 13  2
        [$hd+2] 13  7
        [$hd+3]  0  7
    )
    TexPolyShading( ai [$hd+4] [$hd+7] [$hd+3] [$hd+0] )
    Inst_7D
    TexPoly( ai
        [$hd+4]  0  2
        [$hd+7] 34  2
        [$hd+3] 34  7
        [$hd+0]  0  7
    )
    Poly( a  [$hd+4] [$hd+5] [$hd+6] [$hd+7] )
    IfVarRange( :hdbe 382 $ch  [$he-1] )
    IfVarRange( :hdbe 37E ipt[[$hd+3]x] ipt[[$hd+0]x] )
    IfVarRange( :hdbe 386 ipt[[$hd+1]y] ipt[[$hd+0]y] )
    SetVar( 284 6 )
:hdbe
    Return

:hd_top
    VectorJump( :hdt_nolight a [$hd+4] [$hd+5] [$hd+6] [$hd+7] )
    SurfaceColor( 3 F0 )
    Bitmap( concrete.r8 0 0 0 0 )
    Smoothing( 1 )
    Inst_7D
    TexPoly( at [$hd+4] [$hd+5] [$hd+6] [$hd+7] )
    Bitmap( helo1.r8 0 0 0 0 )
    Inst_7D
    TexPoly( atl 128 128 255 255
        [$hd+14] [$hd+15] [$hd+8] [$hd+9]
        [$hd+10] [$hd+11] [$hd+12] [$hd+13]
    )
    Smoothing( 0 )
    IfVarAnd( :hdt_nolight 28C 6 )
    IfVarRange( :hdt_nolight 33B 0 800 )
    LineColor( 15 F0 )

    DotLine(
        ipt[ [$hd+14] x] ipt[[$hd+14] z] ipt[[$hd+14] y]
        ipt[ [$hd+15] x] ipt[[$hd+15] z] ipt[[$hd+15] y]
        3 )
    DotLine(
        ipt[ [$hd+15] x] ipt[[$hd+15] z] ipt[[$hd+15] y]
        ipt[ [$hd+ 8] x] ipt[[$hd+ 8] z] ipt[[$hd+ 8] y]
        5 )
    DotLine(
        ipt[ [$hd+ 8] x] ipt[[$hd+ 8] z] ipt[[$hd+ 8] y]
        ipt[ [$hd+ 9] x] ipt[[$hd+ 9] z] ipt[[$hd+ 9] y]
        3 )
    DotLine(
        ipt[ [$hd+ 9] x] ipt[[$hd+ 9] z] ipt[[$hd+ 9] y]
        ipt[ [$hd+10] x] ipt[[$hd+10] z] ipt[[$hd+10] y]
        4 )
    DotLine(
        ipt[ [$hd+10] x] ipt[[$hd+10] z] ipt[[$hd+10] y]
        ipt[ [$hd+11] x] ipt[[$hd+11] z] ipt[[$hd+11] y]
        3 )
    DotLine(
        ipt[ [$hd+11] x] ipt[[$hd+11] z] ipt[[$hd+11] y]
        ipt[ [$hd+12] x] ipt[[$hd+12] z] ipt[[$hd+12] y]
        4 )
    DotLine(
        ipt[ [$hd+12] x] ipt[[$hd+12] z] ipt[[$hd+12] y]
        ipt[ [$hd+13] x] ipt[[$hd+13] z] ipt[[$hd+13] y]
        4 )
    DotLine(
        ipt[ [$hd+13] x] ipt[[$hd+13] z] ipt[[$hd+13] y]
        ipt[ [$hd+14] x] ipt[[$hd+14] z] ipt[[$hd+14] y]
        5 )
    LineColor( F F0 )
    DotPt( [$hd + 4] )
    DotPt( [$hd + 5] )
:hdt_nolight
    Return

    ;--------
    ; tower drawing routine
    ;
:tower
    VectorJump( :tw_low m 0 32767 0  $he )
    Call ( :tw_base )
    Call ( :derrick )
    Return

:tw_low
    Call ( :derrick )
    Call ( :tw_base )
    Return

:tw_base
    SurfaceColor( 3 F0 )
    Bitmap( side8.r8 0 0 0 0 )
    TexPolyShading( a [$tb+0] [$tb+1] [$tb+5] [$tb+4] )
    Inst_7D
    TexPoly( a
        [$tb+5]  0  2
        [$tb+4] 10  2
        [$tb+0] 10  7
        [$tb+1]  0  7
    )
    TexPolyShading( a [$tb+6] [$tb+5] [$tb+1] [$tb+2] )
    Inst_7D
    TexPoly( a
        [$tb+6]  0  2
        [$tb+5] 10  2
        [$tb+1] 10  7
        [$tb+2]  0  7
    )
    TexPolyShading( a [$tb+7] [$tb+6] [$tb+2] [$tb+3] )
    Inst_7D
    TexPoly( a
        [$tb+7]  0  2
        [$tb+6] 10  2
        [$tb+2] 10  7
        [$tb+3]  0  7
    )
    TexPolyShading( a  [$tb+4] [$tb+7] [$tb+3] [$tb+0] )
    Inst_7D
    TexPoly( a
        [$tb+4]  0  2
        [$tb+7] 10  2
        [$tb+3] 10  7
        [$tb+0]  0  7
    )
    VectorJump( :twbe a [$tb+4] [$tb+5] [$tb+6] [$tb+7] )
    IfVarRange( :twb2 346 4 4 )
    Bitmap( concrete.r8 0 0 0 0 )
:twb2
    Poly( a  [$tb+4] [$tb+5] [$tb+6] [$tb+7] )
    IfVarAnd( :twbe 28C 6 )
    IfVarRange( :bdg2e 33B 0 900 )
    LineColor( F F0 )
    DotPt( [$tb + 4] )
    DotPt( [$tb + 5] )
    DotPt( [$tb + 6] )
    DotPt( [$tb + 7] )
:twbe
    Return

    ; this is the drilling derrick
:derrick
    IfVarRange( :dr_simple 346  2 4 )
    Monitor3D( :dr1 -4 4  int[$hm/8] int[ipt[[$dr+32] z]/8]  -4 4 )
    SetVar( 284 6 )
:dr1
    VectorJump( :dr2 m 0 -32767 0 [$he + 160 * -1] )
    Call( :drs4 )
    Call( :drs3 )
    Call( :drs2 )
    Call( :drs1 )
    Return

:dr2
    VectorJump( :dr3 m 0 -32767 0 [$he + 200 * -1] )
    Call( :drs4 )
    Call( :drs3 )
    Call( :drs1 )
    Call( :drs2 )
    Return

:dr3
    VectorJump( :dr4 m 0 -32767 0 [$he + 360 * -1] )
    Call( :drs4 )
    Call( :drs1 )
    Call( :drs2 )
    Call( :drs3 )
    Return

:dr4
    Call( :drs1 )
    Call( :drs2 )
    Call( :drs3 )
    Call( :drs4 )
    Return

:dr_simple
    LineColor( 1 F0 )
    MoveToPt( [$dr+ 0] )
    DrawToPt( [$dr+32] )
    MoveToPt( [$dr+ 1] )
    DrawToPt( [$dr+32] )
    MoveToPt( [$dr+ 2] )
    DrawToPt( [$dr+32] )
    MoveToPt( [$dr+ 3] )
    DrawToPt( [$dr+32] )
    IfVarAnd( :no_drl 282 F0F )
    LineColor( F F0 )
    SurfaceColor( F F0 )
    IfVarRange( :dr_far 33B 0 2000 )
    BigDot( 512 ipt[[$dr+32]x] ipt[[$dr+32]z] ipt[[$dr+32]y] )
    Jump( :no_drl )
:dr_far
    DotPt( [$dr+32] )
:no_drl
    Return

:drs1
    ; lower derrick structure
    LineColor( 2 F0 )
    MoveToPt( [$dr + 0] )
    DrawToPt( [$dr + 5] [$dr + 1] [$dr + 6] )
    MoveToPt( [$dr + 1] )
    DrawToPt( [$dr + 7] [$dr + 2] [$dr + 8] )
    MoveToPt( [$dr + 2] )
    DrawToPt( [$dr + 9] [$dr + 3] [$dr +10] )
    MoveToPt( [$dr + 3] )
    DrawToPt( [$dr +11] [$dr + 0] [$dr + 4] )
    Return

:drs2
    SurfaceColor( 2 F0 )
    Poly( a  [$dr+12] [$dr+13] [$dr+14] [$dr+15] )
    Poly( a  [$dr+ 4] [$dr+ 6] [$dr+13] [$dr+12] )
    Poly( a  [$dr+ 6] [$dr+ 8] [$dr+14] [$dr+13] )
    Poly( a  [$dr+ 8] [$dr+10] [$dr+15] [$dr+14] )
    Poly( a  [$dr+10] [$dr+ 4] [$dr+12] [$dr+15] )
    Poly( ad [$dr+ 4] [$dr+ 6] [$dr+ 8] [$dr+10] )
    IfVarAnd  ( :drs2e 28C 6 )
    IfVarRange( :drs2e 33B 0 1200 )
    LineColor( F F0 )
    VectorJump( :drs22 a  [$dr+ 4] [$dr+ 6] [$dr+13] [$dr+12] )
    DotPt( [$dr+12] )
    DotPt( [$dr+13] )
:drs22
    VectorJump( :drs23 a  [$dr+ 6] [$dr+ 8] [$dr+14] [$dr+13] )
    DotPt( [$dr+13] )
    DotPt( [$dr+14] )
:drs23
    VectorJump( :drs24 a  [$dr+ 8] [$dr+10] [$dr+15] [$dr+14] )
    DotPt( [$dr+14] )
    DotPt( [$dr+15] )
:drs24
    VectorJump( :drs2e a  [$dr+10] [$dr+ 4] [$dr+12] [$dr+15] )
    DotPt( [$dr+12] )
    DotPt( [$dr+15] )
:drs2e
    Return

:drs3
    ; upper derrick structure
    LineColor( 2 F0 )
    MoveToPt( [$dr +24] )
    DrawToPt( [$dr +16] [$dr +23] [$dr +24] [$dr +21] [$dr +16] )
    MoveToPt( [$dr +25] )
    DrawToPt( [$dr +17] [$dr +20] [$dr +25] [$dr +22] [$dr +17] )
    MoveToPt( [$dr +26] )
    DrawToPt( [$dr +18] [$dr +21] [$dr +26] [$dr +23] [$dr +18] )
    MoveToPt( [$dr +27] )
    DrawToPt( [$dr +19] [$dr +22] [$dr +27] [$dr +20] [$dr +19] )
    Return

:drs4
    ; top section of the derrick
    SurfaceColor( 2 F0 )
    Poly( a [$dr+28] [$dr+29] [$dr+30] [$dr+31] )
    LineColor( 5 F0 )
    MoveToPt( [$dr+32] )
    DrawToPt( [$dr+33] )
    IfVarAnd( :no_drsl 282 F0F )
    ; the derrick flash light is on
    SurfaceColor( F F0 )
    LineColor( F F0 )
    IfVarRange( :drs4_far 33B 0 1000 )
    BigDot( 512 ipt[[$dr+33]x] ipt[[$dr+33]z] ipt[[$dr+33]y] )
    Jump( :no_drsl )
:drs4_far
    DotPt( [$dr+33] )
:no_drsl
    SurfaceColor( 2 F0 )
    Poly( a  [$dr+24] [$dr+25] [$dr+29] [$dr+28] )
    Poly( a  [$dr+25] [$dr+26] [$dr+30] [$dr+29] )
    Poly( a  [$dr+26] [$dr+27] [$dr+31] [$dr+30] )
    Poly( a  [$dr+27] [$dr+24] [$dr+28] [$dr+31] )
    Poly( ad [$dr+24] [$dr+25] [$dr+26] [$dr+27] )
    ; obstacle lights
    IfVarAnd  ( :drs4e 28C 6 )
    IfVarRange( :drs4e 33B 0 1200 )
    LineColor( F F0 )
    VectorJump( :drs42 a  [$dr+24] [$dr+25] [$dr+29] [$dr+28] )
    DotPt( [$dr+28] )
    DotPt( [$dr+29] )
:drs42
    VectorJump( :drs43 a  [$dr+25] [$dr+26] [$dr+30] [$dr+29] )
    DotPt( [$dr+29] )
    DotPt( [$dr+30] )
:drs43
    VectorJump( :drs44 a  [$dr+26] [$dr+27] [$dr+31] [$dr+30] )
    DotPt( [$dr+30] )
    DotPt( [$dr+31] )
:drs44
    VectorJump( :drs4e a  [$dr+27] [$dr+24] [$dr+28] [$dr+31] )
    DotPt( [$dr+28] )
    DotPt( [$dr+31] )
:drs4e
    Return
;
; ---- end of derrick section ----

;
;   Subroutine for drawing all the columns
;
:columns
    VectorJump( :co_s   m 0 0 32767   0 )
    VectorJump( :co_nw  m 32767 0 0   0 )
    ; ne sector
    TransformCall( :big_col -240 0 -272 0 0  0 0  0 0 ) ; 5
    TransformCall( :sml_col -240 0  -92 0 0  0 0  0 0 ) ; 6
    TransformCall( :sml_col -240 0   92 0 0  0 0  0 0 ) ; 7
    TransformCall( :big_col -240 0  272 0 0  0 0  0 0 ) ; 8
    TransformCall( :big_col  240 0 -272 0 0  0 0  0 0 ) ; 4
    TransformCall( :sml_col  240 0  -92 0 0  0 0  0 0 ) ; 3
    Call( :riser )
    TransformCall( :sml_col  240 0   92 0 0  0 0  0 0 ) ; 2
    TransformCall( :big_col  240 0  272 0 0  0 0  0 0 ) ; 1
    Return

:co_nw
    ; nw sector
    TransformCall( :big_col  240 0 -272 0 0  0 0  0 0 ) ; 4
    TransformCall( :sml_col  240 0  -92 0 0  0 0  0 0 ) ; 3
    TransformCall( :sml_col  240 0   92 0 0  0 0  0 0 ) ; 2
    TransformCall( :big_col  240 0  272 0 0  0 0  0 0 ) ; 1
    TransformCall( :big_col -240 0 -272 0 0  0 0  0 0 ) ; 5
    TransformCall( :sml_col -240 0  -92 0 0  0 0  0 0 ) ; 6
    Call( :riser )
    TransformCall( :sml_col -240 0   92 0 0  0 0  0 0 ) ; 7
    TransformCall( :big_col -240 0  272 0 0  0 0  0 0 ) ; 8
    Return

:co_s
    VectorJump( :co_sw   m 32767 0 0      0 )
    ; se sector
    TransformCall( :big_col -240 0  272 0 0  0 0  0 0 ) ; 8
    TransformCall( :sml_col -240 0   92 0 0  0 0  0 0 ) ; 7
    TransformCall( :sml_col -240 0  -92 0 0  0 0  0 0 ) ; 6
    TransformCall( :big_col -240 0 -272 0 0  0 0  0 0 ) ; 5
    TransformCall( :big_col  240 0  272 0 0  0 0  0 0 ) ; 1
    TransformCall( :sml_col  240 0   92 0 0  0 0  0 0 ) ; 2
    Call( :riser )
    TransformCall( :sml_col  240 0  -92 0 0  0 0  0 0 ) ; 3
    TransformCall( :big_col  240 0 -272 0 0  0 0  0 0 ) ; 4
    Return

:co_sw
    ; sw sector
    TransformCall( :big_col  240 0  272 0 0  0 0  0 0 ) ; 1
    TransformCall( :sml_col  240 0   92 0 0  0 0  0 0 ) ; 2
    TransformCall( :sml_col  240 0  -92 0 0  0 0  0 0 ) ; 3
    TransformCall( :big_col  240 0 -272 0 0  0 0  0 0 ) ; 4
    TransformCall( :big_col -240 0  272 0 0  0 0  0 0 ) ; 8
    TransformCall( :sml_col -240 0   92 0 0  0 0  0 0 ) ; 7
    Call( :riser )
    TransformCall( :sml_col -240 0  -92 0 0  0 0  0 0 ) ; 6
    TransformCall( :big_col -240 0 -272 0 0  0 0  0 0 ) ; 5
    Return
;
;   This is one of the big columns
;
:big_col
    VecPoints( xy 0
        [round[-$rb]]           0    0                       ; 0
        [round[-$rb * cos[45]]] 0    [round[ $rb * sin[45]]] ; 1
        0                       0    [round[ $rb]]           ; 2
        [round[ $rb * cos[45]]] 0    [round[ $rb * sin[45]]] ; 3
        [round[ $rb]]           0    0                       ; 4
        [round[ $rb * cos[45]]] 0    [round[-$rb * sin[45]]] ; 5
        0                       0    [round[-$rb]]           ; 6
        [round[-$rb * cos[45]]] 0    [round[-$rb * sin[45]]] ; 7

        [round[-$rb]]           $ch  0                       ; 8
        [round[-$rb * cos[45]]] $ch  [round[ $rb * sin[45]]] ; 9
        0                       $ch  [round[ $rb]]           ; 10
        [round[ $rb * cos[45]]] $ch  [round[ $rb * sin[45]]] ; 11
        [round[ $rb]]           $ch  0                       ; 12
        [round[ $rb * cos[45]]] $ch  [round[-$rb * sin[45]]] ; 13
        0                       $ch  [round[-$rb]]           ; 14
        [round[-$rb * cos[45]]] $ch  [round[-$rb * sin[45]]] ; 15
    )
    ShadedColor( 2 F0 )
    ShadedPoly( a 0 1  9  8 )
    ShadedPoly( a 1 2 10  9 )
    ShadedPoly( a 2 3 11 10 )
    ShadedPoly( a 3 4 12 11 )
    ShadedPoly( a 4 5 13 12 )
    ShadedPoly( a 5 6 14 13 )
    ShadedPoly( a 6 7 15 14 )
    ShadedPoly( a 7 0  8 15 )
    IfVarRange ( :bc2 346 2 4 )
    IfVarRange ( :bc2 382 0 $ch )
    SenseBorder( :bc2 ipt[0 x] ipt[0 y] ipt[2 x] ipt[2 y]
                      ipt[4 x] ipt[4 y] ipt[6 x] ipt[6 y] )
    SetVar( 284 6 )
:bc2
    return

;
;   This is one of the small columns
;
:sml_col
    VecPoints( xy 0
        [round[-$rs]]           0    0                       ; 0
        [round[-$rs * cos[45]]] 0    [round[ $rs * sin[45]]] ; 1
        0                       0    [round[ $rs]]           ; 2
        [round[ $rs * cos[45]]] 0    [round[ $rs * sin[45]]] ; 3
        [round[ $rs]]           0    0                       ; 4
        [round[ $rs * cos[45]]] 0    [round[-$rs * sin[45]]] ; 5
        0                       0    [round[-$rs]]           ; 6
        [round[-$rs * cos[45]]] 0    [round[-$rs * sin[45]]] ; 7

        [round[-$rs]]           $ch  0                       ; 8
        [round[-$rs * cos[45]]] $ch  [round[ $rs * sin[45]]] ; 9
        0                       $ch  [round[ $rs]]           ; 10
        [round[ $rs * cos[45]]] $ch  [round[ $rs * sin[45]]] ; 11
        [round[ $rs]]           $ch  0                       ; 12
        [round[ $rs * cos[45]]] $ch  [round[-$rs * sin[45]]] ; 13
        0                       $ch  [round[-$rs]]           ; 14
        [round[-$rs * cos[45]]] $ch  [round[-$rs * sin[45]]] ; 15
    )
    ShadedColor( 2 F0 )
    ShadedPoly( a 0 1  9  8 )
    ShadedPoly( a 1 2 10  9 )
    ShadedPoly( a 2 3 11 10 )
    ShadedPoly( a 3 4 12 11 )
    ShadedPoly( a 4 5 13 12 )
    ShadedPoly( a 5 6 14 13 )
    ShadedPoly( a 6 7 15 14 )
    ShadedPoly( a 7 0  8 15 )
    IfVarRange ( :sc2 346 2 4 )
    IfVarRange ( :sc2 382 0 $ch )
    SenseBorder( :sc2 ipt[0 x] ipt[0 y] ipt[2 x] ipt[2 y]
                      ipt[4 x] ipt[4 y] ipt[6 x] ipt[6 y] )
    SetVar( 284 6 )
:sc2
    Return

;
;   This is the riser
;
:riser
    VecPoints( xy 0
        -2      0       0 ; 0
         0      0       2 ; 1
         2      0       0 ; 2
         0      0      -2 ; 3
        -2      $ch     0 ; 4
         0      $ch     2 ; 5
         2      $ch     0 ; 6
         0      $ch    -2 ; 7
    )
    ShadedColor( 2 F0 )
    ShadedPoly( a 0 1 5 4 )
    ShadedPoly( a 1 2 6 5 )
    ShadedPoly( a 2 3 7 6 )
    ShadedPoly( a 3 0 4 7 )
    IfVarRange ( :rs2 346 2 4 )
    Monitor3D( :rs2 -1 1  0 round[$ch/8]  -1 1 )
    SetVar( 284 6 )
:rs2
    Return

;--------------------------------------------------------
; the crane drawing subroutine
;
:crane
    VecPoints( xy $cr
        [round[-$rc]]           0    0                       ; 0
        [round[-$rc * cos[60]]] 0    [round[ $rc * sin[60]]] ; 1
        [round[ $rc * cos[60]]] 0    [round[ $rc * sin[60]]] ; 2
        [round[ $rc]]           0    0                       ; 3
        [round[ $rc * cos[60]]] 0    [round[-$rc * sin[60]]] ; 4
        [round[-$rc * cos[60]]] 0    [round[-$rc * sin[60]]] ; 5

        [round[-$rc]]           $hb1 0                       ; 6
        [round[-$rc * cos[60]]] $hb1 [round[ $rc * sin[60]]] ; 7
        [round[ $rc * cos[60]]] $hb1 [round[ $rc * sin[60]]] ; 8
        [round[ $rc]]           $hb1 0                       ; 9
        [round[ $rc * cos[60]]] $hb1 [round[-$rc * sin[60]]] ; 10
        [round[-$rc * cos[60]]] $hb1 [round[-$rc * sin[60]]] ; 11
    )
    Points( [$cr + 12]
        -16     [$hb1+0]        -16 ; 12
        -16     [$hb1+0]         16 ; 13
         16     [$hb1+0]         16 ; 14
         16     [$hb1+0]        -16 ; 15
          0     [$hb1+32]         0 ; 16
    )
    VectorJump( :cr2 m 0 32767 0  $hb1 )
    Call( :crb )
    TransformCall( :crt 0 $hb1 0  0 314  0 0  0 0 )
    Return

:cr2
    TransformCall( :crt 0 $hb1 0  0 314  0 0  0 0 )
    Call( :crb )
    Return

:crb
    ; crane base
    ShadedColor( 3 F0 )
    ShadedPoly( a [$cr+0] [$cr+1] [$cr+7]  [$cr+6] )
    ShadedPoly( a [$cr+1] [$cr+2] [$cr+8]  [$cr+7] )
    ShadedPoly( a [$cr+2] [$cr+3] [$cr+9]  [$cr+8] )
    ShadedPoly( a [$cr+3] [$cr+4] [$cr+10] [$cr+9] )
    ShadedPoly( a [$cr+4] [$cr+5] [$cr+11] [$cr+10] )
    ShadedPoly( a [$cr+5] [$cr+0] [$cr+6]  [$cr+11] )
    Poly( a [$cr+6] [$cr+7] [$cr+8] [$cr+9] [$cr+10] [$cr+11] )
    Return

:crt
    Points( [$cr + 17]
        -14     0          0 ; 17 Ausleger
        -14     0         80 ; 18
        -14     0        160 ; 19
        -14     0        240 ; 20
        -14     0        320 ; 21
        -14     0        400 ; 22
         14     0          0 ; 23
         14     0         80 ; 24
         14     0        160 ; 25
         14     0        240 ; 26
         14     0        320 ; 27
         14     0        400 ; 28
          0     14        40 ; 29
          0     14       120 ; 30
          0     14       200 ; 31
          0     14       280 ; 32
          0     14       360 ; 33
    )

    LineColor( 2 F0 )
    MoveToPt( [$cr+12] )
    DrawToPt( [$cr+16] )
    MoveToPt( [$cr+13] )
    DrawToPt( [$cr+16] )
    MoveToPt( [$cr+14] )
    DrawToPt( [$cr+16] )
    MoveToPt( [$cr+15] )
    DrawToPt( [$cr+16] )

;    LineColor( 4 F0 )
    MoveToPt( [$cr+17] )
    DrawToPt( [$cr+22] )
    DrawToPt( [$cr+28] )
    DrawToPt( [$cr+23] )

    IfVRes( :crt2 3 1 )
    LineColor( 2 F0 )
    MoveToPt( [$cr+17] )
    DrawToPt( [$cr+29] )
    DrawToPt( [$cr+24] )
    DrawToPt( [$cr+30] )
    DrawToPt( [$cr+19] )
    DrawToPt( [$cr+31] )
    DrawToPt( [$cr+26] )
    DrawToPt( [$cr+32] )
    DrawToPt( [$cr+21] )
    DrawToPt( [$cr+33] )
    DrawToPt( [$cr+28] )

    MoveToPt( [$cr+23] )
    DrawToPt( [$cr+29] )
    DrawToPt( [$cr+18] )
    DrawToPt( [$cr+30] )
    DrawToPt( [$cr+25] )
    DrawToPt( [$cr+31] )
    DrawToPt( [$cr+20] )
    DrawToPt( [$cr+32] )
    DrawToPt( [$cr+27] )
    DrawToPt( [$cr+33] )
    DrawToPt( [$cr+22] )

    LineColor( 5 F0 )
    Brightness( 66 )
    MoveToPt( [$cr+29] )
    DrawToPt( [$cr+33] )

:crt2
    LineColor( 0 F0 )
    MoveToPt( [$cr+16] )
    DrawToPt( [$cr+33] )
    ; obstacle lights
    IfVarAnd( :crt2nl 28C 6 )
    IfVarRange( :crt2nl 33B 0 800 )
    LineColor( F F0 )
    DotPt( [$cr+16] )
    DotPt( [$cr+33] )
:crt2nl
    Return
;
;--- end of crane --------

:end
EndA

;-----------------------------------------------------------------
;
;   hard surface definition
;
Area16( a %1 %2 5.556 5.556 )
    Refpoint( abs : 0.125  %1 %2 )
    mif( %5 )
    RotatedCall( :A  0 0 %5 )
    melse
    Call( :A )
    mifend
    Jump( : )

:A
    SenseBorder( :next          ; helo pad
         -56  -324   -112  -404 ;  8  9
        -272  -404   -332  -324 ; 10 11
        -332  -204   -272  -108 ; 12 13
        -112  -108    -56  -164 ; 14 15
    )
    SetElevation( [round[ $he / 8 ]] )
    Return

:next
    SenseBorder( :next3
         260   352 ; 16
         300   312 ; 19
         300  -312 ; 20
         260  -352 ; 23
        -260  -352 ; 24
        -300  -312 ; 27
        -300   312 ; 28
        -260   352 ; 31
    )
    SetElevation( [round[ $hm / 8 ]] ) ; main deck level
    ;
    SenseBorder( :next2
         272  -108    272  -324 ;  4  5
        -272  -324   -272  -108 ;  6  7
    )
    SetElevation( [round[ $he / 8 ]] ) ; building 1
:next2
    SenseBorder( :next3
         200   320    200   160 ; 0 1
        -200   160   -200   320 ; 2 3
    )
    SetElevation( [round[[$hm + 32] / 8]] ) ; small building
:next3
    Return

End16
